@@ -1,18 +1,20 @@
 import asyncio
 import signal
 import socket
+import warnings
 from abc import ABC, abstractmethod
 from typing import Any, List, Optional, Set
 
 from yarl import URL
 
+from .typedefs import PathLike
 from .web_app import Application
 from .web_server import Server
 
 try:
     from ssl import SSLContext
 except ImportError:
-    SSLContext = object  # type: ignore
+    SSLContext = object  # type: ignore[misc,assignment]
 
 
 __all__ = (
@@ -37,7 +39,7 @@ def _raise_graceful_exit() -> None:
 
 
 class BaseSite(ABC):
-    __slots__ = ("_runner", "_shutdown_timeout", "_ssl_context", "_backlog", "_server")
+    __slots__ = ("_runner", "_ssl_context", "_backlog", "_server")
 
     def __init__(
         self,
@@ -45,15 +47,18 @@ class BaseSite(ABC):
         *,
         shutdown_timeout: float = 60.0,
         ssl_context: Optional[SSLContext] = None,
-        backlog: int = 128
+        backlog: int = 128,
     ) -> None:
         if runner.server is None:
             raise RuntimeError("Call runner.setup() before making a site")
+        if shutdown_timeout != 60.0:
+            msg = "shutdown_timeout should be set on BaseRunner"
+            warnings.warn(msg, DeprecationWarning, stacklevel=2)
+            runner._shutdown_timeout = shutdown_timeout
         self._runner = runner
-        self._shutdown_timeout = shutdown_timeout
         self._ssl_context = ssl_context
         self._backlog = backlog
-        self._server = None  # type: Optional[asyncio.AbstractServer]
+        self._server: Optional[asyncio.AbstractServer] = None
 
     @property
     @abstractmethod
@@ -66,16 +71,9 @@ class BaseSite(ABC):
 
     async def stop(self) -> None:
         self._runner._check_site(self)
-        if self._server is None:
-            self._runner._unreg_site(self)
-            return  # not started yet
-        self._server.close()
-        # named pipes do not have wait_closed property
-        if hasattr(self._server, "wait_closed"):
-            await self._server.wait_closed()
-        await self._runner.shutdown()
-        assert self._runner.server
-        await self._runner.server.shutdown(self._shutdown_timeout)
+        if self._server is not None:  # Maybe not started yet
+            self._server.close()
+
         self._runner._unreg_site(self)
 
 
@@ -92,7 +90,7 @@ class TCPSite(BaseSite):
         ssl_context: Optional[SSLContext] = None,
         backlog: int = 128,
         reuse_address: Optional[bool] = None,
-        reuse_port: Optional[bool] = None
+        reuse_port: Optional[bool] = None,
     ) -> None:
         super().__init__(
             runner,
@@ -110,7 +108,7 @@ class TCPSite(BaseSite):
     @property
     def name(self) -> str:
         scheme = "https" if self._ssl_context else "http"
-        host = "0.0.0.0" if self._host is None else self._host
+        host = "0.0.0.0" if not self._host else self._host
         return str(URL.build(scheme=scheme, host=host, port=self._port))
 
     async def start(self) -> None:
@@ -135,11 +133,11 @@ class UnixSite(BaseSite):
     def __init__(
         self,
         runner: "BaseRunner",
-        path: str,
+        path: PathLike,
         *,
         shutdown_timeout: float = 60.0,
         ssl_context: Optional[SSLContext] = None,
-        backlog: int = 128
+        backlog: int = 128,
     ) -> None:
         super().__init__(
             runner,
@@ -152,7 +150,7 @@ class UnixSite(BaseSite):
     @property
     def name(self) -> str:
         scheme = "https" if self._ssl_context else "http"
-        return "{}://unix:{}:".format(scheme, self._path)
+        return f"{scheme}://unix:{self._path}:"
 
     async def start(self) -> None:
         await super().start()
@@ -160,7 +158,10 @@ class UnixSite(BaseSite):
         server = self._runner.server
         assert server is not None
         self._server = await loop.create_unix_server(
-            server, self._path, ssl=self._ssl_context, backlog=self._backlog
+            server,
+            self._path,
+            ssl=self._ssl_context,
+            backlog=self._backlog,
         )
 
 
@@ -171,9 +172,11 @@ class NamedPipeSite(BaseSite):
         self, runner: "BaseRunner", path: str, *, shutdown_timeout: float = 60.0
     ) -> None:
         loop = asyncio.get_event_loop()
-        if not isinstance(loop, asyncio.ProactorEventLoop):  # type: ignore
+        if not isinstance(
+            loop, asyncio.ProactorEventLoop  # type: ignore[attr-defined]
+        ):
             raise RuntimeError(
-                "Named Pipes only available in proactor" "loop under windows"
+                "Named Pipes only available in proactor loop under windows"
             )
         super().__init__(runner, shutdown_timeout=shutdown_timeout)
         self._path = path
@@ -187,7 +190,9 @@ class NamedPipeSite(BaseSite):
         loop = asyncio.get_event_loop()
         server = self._runner.server
         assert server is not None
-        _server = await loop.start_serving_pipe(server, self._path)  # type: ignore
+        _server = await loop.start_serving_pipe(  # type: ignore[attr-defined]
+            server, self._path
+        )
         self._server = _server[0]
 
 
@@ -201,7 +206,7 @@ class SockSite(BaseSite):
         *,
         shutdown_timeout: float = 60.0,
         ssl_context: Optional[SSLContext] = None,
-        backlog: int = 128
+        backlog: int = 128,
     ) -> None:
         super().__init__(
             runner,
@@ -212,7 +217,7 @@ class SockSite(BaseSite):
         self._sock = sock
         scheme = "https" if self._ssl_context else "http"
         if hasattr(socket, "AF_UNIX") and sock.family == socket.AF_UNIX:
-            name = "{}://unix:{}:".format(scheme, sock.getsockname())
+            name = f"{scheme}://unix:{sock.getsockname()}:"
         else:
             host, port = sock.getsockname()[:2]
             name = str(URL.build(scheme=scheme, host=host, port=port))
@@ -233,13 +238,20 @@ class SockSite(BaseSite):
 
 
 class BaseRunner(ABC):
-    __slots__ = ("_handle_signals", "_kwargs", "_server", "_sites")
+    __slots__ = ("_handle_signals", "_kwargs", "_server", "_sites", "_shutdown_timeout")
 
-    def __init__(self, *, handle_signals: bool = False, **kwargs: Any) -> None:
+    def __init__(
+        self,
+        *,
+        handle_signals: bool = False,
+        shutdown_timeout: float = 60.0,
+        **kwargs: Any,
+    ) -> None:
         self._handle_signals = handle_signals
         self._kwargs = kwargs
-        self._server = None  # type: Optional[Server]
-        self._sites = []  # type: List[BaseSite]
+        self._server: Optional[Server] = None
+        self._sites: List[BaseSite] = []
+        self._shutdown_timeout = shutdown_timeout
 
     @property
     def server(self) -> Optional[Server]:
@@ -247,11 +259,11 @@ class BaseRunner(ABC):
 
     @property
     def addresses(self) -> List[Any]:
-        ret = []  # type: List[Any]
+        ret: List[Any] = []
         for site in self._sites:
             server = site._server
             if server is not None:
-                sockets = server.sockets
+                sockets = server.sockets  # type: ignore[attr-defined]
                 if sockets is not None:
                     for sock in sockets:
                         ret.append(sock.getsockname())
@@ -276,24 +288,28 @@ class BaseRunner(ABC):
 
     @abstractmethod
     async def shutdown(self) -> None:
-        pass  # pragma: no cover
+        """Call any shutdown hooks to help server close gracefully."""
 
     async def cleanup(self) -> None:
-        loop = asyncio.get_event_loop()
-
-        if self._server is None:
-            # no started yet, do nothing
-            return
-
         # The loop over sites is intentional, an exception on gather()
         # leaves self._sites in unpredictable state.
         # The loop guaranties that a site is either deleted on success or
         # still present on failure
         for site in list(self._sites):
             await site.stop()
+
+        if self._server:  # If setup succeeded
+            # Yield to event loop to ensure incoming requests prior to stopping the sites
+            # have all started to be handled before we proceed to close idle connections.
+            await asyncio.sleep(0)
+            self._server.pre_shutdown()
+            await self.shutdown()
+            await self._server.shutdown(self._shutdown_timeout)
         await self._cleanup_server()
+
         self._server = None
         if self._handle_signals:
+            loop = asyncio.get_running_loop()
             try:
                 loop.remove_signal_handler(signal.SIGINT)
                 loop.remove_signal_handler(signal.SIGTERM)
@@ -311,22 +327,16 @@ class BaseRunner(ABC):
 
     def _reg_site(self, site: BaseSite) -> None:
         if site in self._sites:
-            raise RuntimeError(
-                "Site {} is already registered in runner {}".format(site, self)
-            )
+            raise RuntimeError(f"Site {site} is already registered in runner {self}")
         self._sites.append(site)
 
     def _check_site(self, site: BaseSite) -> None:
         if site not in self._sites:
-            raise RuntimeError(
-                "Site {} is not registered in runner {}".format(site, self)
-            )
+            raise RuntimeError(f"Site {site} is not registered in runner {self}")
 
     def _unreg_site(self, site: BaseSite) -> None:
         if site not in self._sites:
-            raise RuntimeError(
-                "Site {} is not registered in runner {}".format(site, self)
-            )
+            raise RuntimeError(f"Site {site} is not registered in runner {self}")
         self._sites.remove(site)
 
 
